#pragma once

#include <functional>
#include <map>

#include <QGeoCoordinate>
#include <QObject>

#include "Geometry/WimaAreaData.h"
#include "Geometry/WimaCorridorData.h"
#include "Geometry/WimaJoinedAreaData.h"
#include "Geometry/WimaMeasurementAreaData.h"
#include "Geometry/WimaServiceAreaData.h"
#include "MissionItem.h"

class WimaPlanData : public QObject {
  Q_OBJECT
public:
  class Guard {
  public:
    Guard(std::function<void(void)> fun) : fun_(fun) {}
    ~Guard() { fun_(); }

  private:
    std::function<void(void)> fun_;
  };

  WimaPlanData(QObject *parent = nullptr);
  WimaPlanData(const WimaPlanData &other, QObject *parent = nullptr);
  WimaPlanData &operator=(const WimaPlanData &other);

  // Member Methodes
  void set(const WimaJoinedAreaData &areaData);
  void set(const WimaServiceAreaData &areaData);
  void set(const WimaCorridorData &areaData);
  void set(const WimaMeasurementAreaData &areaData);
  void clear();

  const WimaJoinedAreaData &joinedArea() const;
  const WimaServiceAreaData &serviceArea() const;
  const WimaCorridorData &corridor() const;
  const WimaMeasurementAreaData &measurementArea() const;

  void startEditing();
  void stopEditing();
  Guard guard();

  bool operator==(const WimaPlanData &other) const;
  bool operator!=(const WimaPlanData &other) const;

signals:
  void joinedAreaChanged();
  void serviceAreaChanged();
  void corridorChanged();
  void measurementAreaChanged();

private:
  void emitJoinedAreaChanged();
  void emitServiceAreaChanged();
  void emitCorridorChanged();
  void emitMeasurementAreaChanged();

  WimaJoinedAreaData _joinedArea;
  WimaServiceAreaData _serviceArea;
  WimaCorridorData _corridor;
  WimaMeasurementAreaData _measurementArea;

  std::map<int, std::function<void(void)>> _queuedSignals;

  bool _editing;
};
